//----------------------------------------------------------------------------
// Project: PP 01 SD ROM Module
// Author:  RomBor, rombor18 <at> gmail.com
// Date:    02.12. 2018
// MPU:     ATmega328P@16Mhz
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
#ifndef MAIN_H_
#define MAIN_H_
//----------------------------------------------------------------------------
// pin definitions
#define DATABUS_L_DIR                 DDRC
#define DATABUS_L_DATA                PORTC
#define DATABUS_L_MASK                0x0F
#define DATABUS_H_DIR                 DDRD
#define DATABUS_H_DATA                PORTD
#define DATABUS_H_MASK                0xF0

#define MEM_A14_DIR                   DDRC
#define MEM_A14_PORT                  PORTC
#define MEM_A14_BIT                   _BV(4)
#define MEM_ALE_DIR                   DDRC
#define MEM_ALE_PORT                  PORTC
#define MEM_ALE_BIT                   _BV(5)
#define MEM_CSW_DIR                   DDRD
#define MEM_CSW_PORT                  PORTD
#define MEM_CSW_BIT                   _BV(2)
#define MEM_WEM_DIR                   DDRD
#define MEM_WEM_PORT                  PORTD
#define MEM_WEM_BIT                   _BV(3)

// macros
#define switch_memory_bank_0()        MEM_A14_PORT &= ~MEM_A14_BIT
#define switch_memory_bank_1()        MEM_A14_PORT |= MEM_A14_BIT
#define is_memory_bank_1()            (MEM_A14_PORT & MEM_A14_BIT)
#define make_ale_pulse()              MEM_ALE_PORT |= MEM_ALE_BIT; _delay_loop_1(1); MEM_ALE_PORT &= ~MEM_ALE_BIT
#define make_wem_pulse()              MEM_WEM_PORT &= ~MEM_WEM_BIT; _delay_loop_1(1); MEM_WEM_PORT |= MEM_WEM_BIT

// USART definitions
#define BAUD                          9600L
#define UBRRVAL                       (F_CPU / (BAUD * 16) - 1)
#if (UBRRVAL > 4095)
  #error "Bad value of macro UBRRVAL"
#endif

// other definitions
#define FW_VERSION                    0x09

#define BUFFER_SIZE                   512
#define ROM_BLOCK_SIZE                16384   // 16 kB
#define MANAGER_SIZE_MAX              12288   // 12 kB
#define MEM_FILE_BUFFER_ADDR          12288
#define MEM_FILE_BUFFER_SIZE          4096

#define WAIT_SYNCHRO_BYTE             50
#define WAIT_COMMAND_TIMEOUT          10
#define WAIT_BYTE_TIMEOUT             5

#define SYNCHRO_BYTE                  0xAA
#define ACK                           0x33
#define NAK                           0x99

#define OP_NO_TRANSFER                0
#define OP_SEND_DATA                  1
#define OP_RECEIVE_DATA               2
#define OP_ERROR                      32
#define OP_NOTHING                    64

#define RESULT_OK                     0         // OK
#define RESULT_FATAL_ERROR            1         // fatal error
#define RESULT_PATH_NAME_TOO_LONG     2         // file/dir name too long
#define RESULT_NOT_FOUND              3         // file/dir not found
#define RESULT_FILE_ACCESS_ERROR      4         // file access/seek error
#define RESULT_FILE_READ_ERROR        5         // file read error
#define RESULT_FILE_WRITE_ERROR       6         // file write error
#define RESULT_FILE_DELETE_ERROR      7         // file delete error
#define RESULT_FILE_NOT_OPENED        8         // file not opened
#define RESULT_FILE_EOF               9         // end of file reached
#define RESULT_FILE_TOO_LONG          10        // file too long
#define RESULT_DISK_FULL              11        // disk full
#define RESULT_DIR_NOT_EMPTY          12        // directory not empty
#define RESULT_FILE_DIR_EXIST         13        // file or directory already exist
#define RESULT_INVALID_NAME           14        // invalid name for file or directory

//----------------------------------------------------------------------------
#endif /* MAIN_H_ */
//----------------------------------------------------------------------------
